from setuptools import find_packages, setup


setup(
    name="Riki Usermanager",
    version="1.0.1",
    description="User Manager utilities for the Riki",
    author="Arseny Poga",
    author_email="pogaa1@nku.edu",
    packages=find_packages(),
)
